//
//  TSPlaceholderTextView.swift
//  Pods
//
//  Created by 100Years on 2025/3/4.
//

open class TSPlaceholderTextView: UITextView {
    
//    // MARK: - 垂直对齐模式
//    public enum VerticalAlignment {
//        case top
//        case center
//        case bottom
//    }
//    
//    public var verticalAlignment: VerticalAlignment = .top {
//        didSet { setNeedsLayout() }
//    }
    
    // Placeholder Label
    public lazy var placeholderLabel: TopLeftLabel = {
        let placeholderLabel = TopLeftLabel()
        placeholderLabel.font = font
        placeholderLabel.textColor = placeholderColor
        placeholderLabel.text = placeholder
        placeholderLabel.numberOfLines = 0
        placeholderLabel.isUserInteractionEnabled = false
        placeholderLabel.textAlignment = .left
        return placeholderLabel
    }()
    
    public var maxLength:Int = 1000 {
        didSet {
            handleTextViewTextCount(textView: self)
        }
    }
    // Placeholder Text
    public  var placeholder: String? {
        didSet {
            placeholderLabel.text = placeholder
        }
    }
    
    // Placeholder Color
    public var placeholderColor: UIColor = .lightGray {
        didSet {
            placeholderLabel.textColor = placeholderColor
        }
    }
    
    // Placeholder Insets
    public var textInsets: UIEdgeInsets = .zero {
        didSet {
            updateTextContainerInset()
        }
    }
    
    // Override text property to manage placeholder visibility
    open override var text: String! {
        didSet {
            updatePlaceholderVisibility()
        }
    }
    
    public var textViewTextChanged:(()->Void)?
    
    // Initializer
    public init(placeholder: String? = nil,
         text: String? = nil,
         font: UIFont? = nil,
         textColor: UIColor = .black,
         backgroundColor: UIColor = .white,
         textInsets: UIEdgeInsets = .zero) {
        super.init(frame: .zero, textContainer: nil)
        
        self.placeholder = placeholder
        self.font = font
        self.textColor = textColor
        self.backgroundColor = backgroundColor
        self.textInsets = textInsets
        
        setupPlaceholder()
        self.text = text
    }
    
    required public init?(coder: NSCoder) {
        super.init(coder: coder)
        setupPlaceholder()
    }
    
    // Setup placeholder label
    private func setupPlaceholder() {
 
        addSubview(placeholderLabel)
        updatePlaceholderVisibility()
        
        // Observe text changes to update placeholder
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(textDidChange(_:)),
                                               name: UITextView.textDidChangeNotification,
                                               object: self)
        
        // Update text container insets
        updateTextContainerInset()
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
    
    // Update placeholder visibility
    private func updatePlaceholderVisibility() {
        placeholderLabel.isHidden = !text.isEmpty
    }
    
    
    @objc private func textDidChange(_ notification: Notification) {
        if let textView = notification.object as? UITextView {
            handleTextViewTextCount(textView: textView)
            textViewTextChanged?()
        }
        updatePlaceholderVisibility()
    }
    
    open func handleTextViewTextCount(textView:UITextView) {
        if textView.text.count > maxLength {
            textView.text = String(textView.text.prefix(maxLength))
        }
    }
    
    // Update textContainerInset and placeholder frame
    private func updateTextContainerInset() {
        textContainerInset = textInsets
        setNeedsLayout()
    }

    
    // Layout placeholder
    open override func layoutSubviews() {
        super.layoutSubviews()

  
//        let textInsets = textCntainerInset
        let placeholderX = textInsets.left + 5 // Add padding for placeholder
        let placeholderY = textInsets.top
        let placeholderWidth = bounds.width - textInsets.left - textInsets.right - 10 // Adjust for padding
        let placeholderHeight = bounds.height - textInsets.top - textInsets.bottom
        
        placeholderLabel.frame = CGRect(x: placeholderX,
                                        y: placeholderY,
                                        width: placeholderWidth,
                                        height: placeholderHeight)
        
//        handleVerticalAlignment()
    }
    
    func handleVerticalAlignment() {
      
//          guard !bounds.isEmpty else { return }
//          
//        
//        let label = UILabel()
//        label.text = text
//        label.font = font
//        label.numberOfLines = 0
//        var height = label.sizeThatFits(CGSize(width: bounds.width, height: .greatestFiniteMagnitude)).height
//
//        
////          let size = sizeThatFits(CGSize(
////              width: bounds.width,
////              height: CGFloat.greatestFiniteMagnitude
////          ))
//          
//          let topInset: CGFloat
//          switch verticalAlignment {
//          case .top:
//              topInset = 0
//          case .center:
//              topInset = max(0, (bounds.height - height) / 2)
//          case .bottom:
//              topInset = max(0, bounds.height - height)
//          }
//          
//          textContainerInset = UIEdgeInsets(
//              top: topInset,
//              left: textContainerInset.left,
//              bottom: textContainerInset.bottom,
//              right: textContainerInset.right
//          )
      }
}


